package com.tendencies.jcr;

import java.io.Serializable;

import javax.jcr.Node;
import javax.jcr.Session;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Create;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.richfaces.event.UploadEvent;
import org.richfaces.model.TreeNodeImpl;
import org.richfaces.model.UploadItem;

import com.tendencies.jcr.connection.RepositoryConnector;
import com.tendencies.jcr.control.ContentManager;
import com.tendencies.jcr.control.NodeManager;
import com.tendencies.jcr.treeview.NodeData;

@Name("jcrbean")
@Scope(ScopeType.SESSION)
public class JcrBean implements Serializable{
	
	private String mensagem = "";

	private boolean updateTreeview = false;
	
	private TreeNodeImpl<NodeData> treeview;
	
	private String stringTreeview;

	private String addClientCNPJ;
	
	private String category;
	private String addCategoryClientCNPJ;

	private String addFilePath;
	private boolean renderFileUpload = false;
	
	private ContentManager contentManager;
	
	@Create
	public void jcrTest () {
		RepositoryConnector repConn = new RepositoryConnector();
		Session jcrSession = null;
		jcrSession = repConn.getSession();
		if (jcrSession != null) mensagem = "CONECTADO!";
		else mensagem = "NÃO CONECTADO!";
		
		Node rootNode = null;
		
		try {
			rootNode = jcrSession.getRootNode();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		if (rootNode != null) mensagem = mensagem + " NÓ RAIZ NA MÃO!";
		else mensagem = mensagem + " SEM O NÓ RAIZ!";
		
		Node clientContent = null;
		
		try {
			clientContent = rootNode.getNode("clientContent[2]");
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		if (clientContent != null) mensagem = mensagem + " NÓ CLIENTCONTENT NA MÃO!";
		else mensagem = mensagem + " SEM O NÓ CLIENTCONTENT!";

		contentManager = new ContentManager(clientContent);
		
		treeview = contentManager.getTree();
		stringTreeview = contentManager.viewContentTree();
		
	}
	
	// METHODS
	
	public void addClient () {
		if (contentManager.addClient(addClientCNPJ)) {
			mensagem = "Cliente adicionado com sucesso!";
			updateTreeview = true;
		}
		else mensagem = "Não foi possível adicionar o cliente!";
		
		addClientCNPJ = "";
	}
	
	public void removeClient (String removeClientCNPJ) {
		if (contentManager.removeClient(removeClientCNPJ)) {
			mensagem = "Cliente removido com sucesso!";
			updateTreeview = true;
		}
		else mensagem = "Não foi possível remover o cliente!";
	}
	
	public void addCategory (String clientCNPJ) {
		if (contentManager.addCategory(clientCNPJ, category)) {
			mensagem = "Categoria adicionada com sucesso!";
			updateTreeview = true;
			addCategoryClientCNPJ = "";
		}
		else mensagem = "Não foi possível adicionar a categoria!";
	}
	
	public void removeCategory (String categoryPath) {
		int slashIndex = categoryPath.indexOf("/");
		String CNPJ = categoryPath.substring(0, slashIndex);
		String category = categoryPath.substring(slashIndex + 1);
		if (contentManager.removeCategory(CNPJ, category)) {
			mensagem = "Categoria removida com sucesso!";
			updateTreeview = true;
		}
		else mensagem = "Não foi possível remover a categoria";
	}
	
	public void listener (UploadEvent event) throws Exception{
		int slashIndex = addFilePath.indexOf("/");
		String CNPJ = addFilePath.substring(0, slashIndex);
		String category = addFilePath.substring(slashIndex + 1);
        UploadItem item = event.getUploadItem();
        String name = item.getFileName();
        String mimeType = getMimeType(name);
        if (mimeType == null) return;
    	if (contentManager.addContent(item.getData(), name, mimeType, CNPJ, category)) {
    		mensagem = "Arquivo adicionado com sucesso!";
    		updateTreeview = true;
    		addFilePath = "";
    	}
        else mensagem = "Não foi possível adicionar o arquivo";
    	
    	treeview = contentManager.getTree();
    	
    }  

	public String getMimeType (String name) {
		int dotIndex = name.lastIndexOf(".");
		if (dotIndex == -1) return null;
		String extension = name.substring(dotIndex + 1);
		if (extension.equals("pdf")) return "application/pdf";
		else if(extension.equals("odt")) return "application/vnd.oasis.opendocument.text";
		else if(extension.equals("ods")) return "application/vnd.oasis.opendocument.spreadsheet";
		else if(extension.equals("jpg")) return "image/jpeg";
		else if(extension.equals("jpeg")) return "image/jpeg";
		else if(extension.equals("gif")) return "image/gif";
		else if(extension.equals("png")) return "image/png";
		else if(extension.equals("dwg")) return "image/vnd.dwg";
		else if(extension.equals("xml")) return "text/xml";
		else return null;
	}
	
	public void removeContent (String contentPath) {
		if (contentManager.removeContent(contentPath)) {
			mensagem = "Arquivo removido com sucesso!";
			updateTreeview = true;
		}
		else mensagem = "Não foi possível remover o arquivo";
	}	

	// GETTERS AND SETTERS

	public String getStringTreeview() {
		return stringTreeview;
	}
	public void setStringTreeview(String stringTreeview) {
		this.stringTreeview = stringTreeview;
	}

	
	public boolean getRenderFileUpload () {
		if (addFilePath.equals("")) renderFileUpload = false;
		else renderFileUpload = true;
		return renderFileUpload;
	}
	public void setRenderFileUpload () {
		if (addFilePath.equals("")) renderFileUpload = false;
		else renderFileUpload = true;
	}

	
	public String getAddFilePath() {
		return addFilePath;
	}
	public void setAddFilePath(String addFilePath) {
		this.addFilePath = addFilePath;
	}
	
	
	public String getAddCategoryClientCNPJ() {
		return addCategoryClientCNPJ;
	}
	public void setAddCategoryClientCNPJ(String addCategoryClientCNPJ) {
		this.addCategoryClientCNPJ = addCategoryClientCNPJ;
	}
	
	
	public String getCategory() {
		return category;
	}
	public void setCategory(String category) {
		this.category = category;
	}	

	
	public TreeNodeImpl<NodeData> getTreeview() {
		if (updateTreeview) {
			treeview = contentManager.getTree();
			stringTreeview = contentManager.viewContentTree();
			updateTreeview = false;
		}
		return treeview;
	}
	public void setTreeview(TreeNodeImpl<NodeData> treeview) {
		this.treeview = treeview;
	}

	
	public String getAddClientCNPJ() {
		return addClientCNPJ;
	}
	public void setAddClientCNPJ(String addClientCNPJ) {
		this.addClientCNPJ = addClientCNPJ;
	}


	public String getMensagem() {
		return mensagem;
	}
	public void setMensagem(String mensagem) {
		this.mensagem = mensagem;
	}
	
}
