package com.tendencies.jcr.control;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import javax.jcr.AccessDeniedException;
import javax.jcr.InvalidItemStateException;
import javax.jcr.ItemExistsException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.ReferentialIntegrityException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.query.InvalidQueryException;
import javax.jcr.query.Query;
import javax.jcr.query.QueryManager;
import javax.jcr.query.QueryResult;
import javax.jcr.version.VersionException;

public class NodeManager {
	
	public static Node getNode (Node node, String path) {
		
		Node caughtNode = null;
		
		try {
			caughtNode = node.getNode(path);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		
		if (caughtNode != null) return caughtNode;
		else return null;
		
	}
	
	public static List<Property> getChildProperties (Node node) {
		
		List<Property> propertyList = new ArrayList<Property>();
		PropertyIterator pi = null;
		
		try {
			pi = node.getProperties();
		} catch (RepositoryException e) {
			e.printStackTrace();
			return null;
		}
		
		while (pi.hasNext()) {
			Property piProp = pi.nextProperty();
			propertyList.add(piProp);
		}
		
		return propertyList;
		
	}

	public static List<Node> getChildNodes (Node node) {
		
		List<Node> nodeList = new ArrayList<Node>();
		NodeIterator ni = null;
		
		try {
			ni = node.getNodes();
		} catch (RepositoryException e) {
			e.printStackTrace();
			return null;
		}
		
		while (ni.hasNext()){
			Node itNode = ni.nextNode();
			nodeList.add(itNode);
		}
		
		return nodeList;
		
	}
	
	public static boolean addNode (Node node, String newNodeName){
		
		Node newNode = null;
		
		try {
			newNode = node.addNode(newNodeName);
		} catch (Exception e) {
			e.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
				return false;
			}
			return false;
		}
		
		try {
			newNode.getSession().save();
		} catch (Exception e) {
			e.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
				return false;
			}
			return false;
		}
		
		return true;
		
	}
	
	public static boolean addNode (Node node, String newNodeName, String nodeType){
		
		Node newNode = null;
		
		try {
			newNode = node.addNode(newNodeName, nodeType);
		} catch (Exception e) {
			e.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
				return false;
			}
			return false;
		}
		
		try {
			newNode.getSession().save();
		} catch (Exception e) {
			e.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
				return false;
			}
			return false;
		}
		
		return true;
		
	}

	public static boolean removeNode (Session session, String nodePathRelativeFromRoot) {
		
		Node rootNode;
		try {
			rootNode = session.getRootNode();
		} catch (RepositoryException e2) {
			e2.printStackTrace();
			return false;
		}
		
		Node nodeToBeRemoved = getNode(rootNode,nodePathRelativeFromRoot);
		
		if(nodeToBeRemoved == null) return false;
		
		try {
			nodeToBeRemoved.remove();
		} catch (Exception e) {
			e.printStackTrace();
			try {
				session.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
				return false;
			}
			return false;
		}
		
		try {
			session.save();
		} catch (Exception e) {
			e.printStackTrace();
			try {
				session.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
				return false;
			}
			return false;
		}
		
		return true;
		
	}
	
	public static String readNodeType (Node node) {
		
		Property nodeType = null;
		String nodeTypeString = null;
		
		try {
			nodeType = node.getProperty("jcr:primaryType");
		} catch (PathNotFoundException e) {
			e.printStackTrace();
			return null;
		} catch (RepositoryException e) {
			e.printStackTrace();
			return null;
		}
		
		try {
			nodeTypeString = nodeType.getString();
		} catch (ValueFormatException e) {
			e.printStackTrace();
			return null;
		} catch (RepositoryException e) {
			e.printStackTrace();
			return null;
		}
		
		return nodeTypeString;
		
	}
	
	public static String readNodeMixinTypes (Node node) {
		
		return readMultiValuedProperty(node, "jcr:mixinTypes");
						
	}
	
	public static String readNodeExoPermissions (Node node) {
		
		return readMultiValuedProperty(node, "exo:permissions");
		
	}

	public static String readMultiValuedProperty (Node node, String propertyName) {

		Property multiValuedProperty = null;
		Value values[];
		String valuesString = "";
		
		try {
			multiValuedProperty = node.getProperty(propertyName);
		} catch (PathNotFoundException e1) {
			e1.printStackTrace();
			return null;
		} catch (RepositoryException e1) {
			e1.printStackTrace();
			return null;
		}
		
		try {
			values = multiValuedProperty.getValues();
		} catch (ValueFormatException e) {
			e.printStackTrace();
			return null;
		} catch (RepositoryException e) {
			e.printStackTrace();
			return null;
		}	

		for (int i = 0; i < values.length; i++) {
			
			if (!valuesString.equals("")) valuesString = valuesString + ", ";
			
			try {
			valuesString = valuesString + values[i].getString();
			} catch (ValueFormatException e) {
				e.printStackTrace();
				return null;
			} catch (IllegalStateException e) {
				e.printStackTrace();
				return null;
			} catch (RepositoryException e) {
				e.printStackTrace();
				return null;
			}
			
		}
		
		return "(" + valuesString + ")";
			
	}
	
	public static String printTree (Node node) {
		
		return printTree(node, 0, "&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp ", "<br/>");
		
	}
	
	public static String printTree (Node node, int depth, String tab, String nextLine) {
		
		String treeString = "";
		
		for (int i = 0; i < depth; i++){
			treeString = treeString + tab;
		}
		
		String rootName;
		
		try {
			rootName = node.getName();
		} catch (RepositoryException e) {
			e.printStackTrace();
			return null;
		}
		
		if (rootName == null || rootName.equals("")) treeString = treeString + "[[ rootNode ]]" + nextLine;
		else treeString = treeString + "[ " + rootName + " ] - " + NodeManager.readNodeType(node) + nextLine;
		
		List<Node> nodeList = getChildNodes(node);
		
		for (Iterator it = nodeList.iterator(); it.hasNext(); ) {
			Node itNode = (Node) it.next();
			treeString = treeString + printTree(itNode, depth + 1, tab, nextLine);
		}
		
		List<Property> propertyList = getChildProperties(node);
		
		for (Iterator it = propertyList.iterator(); it.hasNext(); ) {
			Property itProperty = (Property) it.next();
			for (int i = 0; i <= depth; i++){
				treeString = treeString + tab;
			}
			try {
				treeString = treeString + itProperty.getName() + nextLine;
			} catch (RepositoryException e) {
				e.printStackTrace();
				return null;
			}
		}
		
		return treeString;
		
	}

	public static String getUUID (Node node) {
		
		Property property;
		String UUID = null;
		
		try {
			property = node.getProperty("jcr:uuid");
		} catch (PathNotFoundException e) {
			e.printStackTrace();
			return null;
		} catch (RepositoryException e) {
			e.printStackTrace();
			return null;
		}
		
		try {
			UUID = property.getString();
		} catch (ValueFormatException e) {
			e.printStackTrace();
			return null;
		} catch (RepositoryException e) {
			e.printStackTrace();
			return null;
		}
		
		return UUID;		
			
	}
	
	public static boolean isMultiValued (Property property){
		
		try {
			property.getValues();
		} catch (ValueFormatException e) {
			return false;
		} catch (RepositoryException e) {
			e.printStackTrace();
			return false;
		}
		
		return true;
		
	}
	
	public static void exportDocView (String nodePath, Session session, String fileAddress) {
		
		File file = new File(fileAddress);
		
		if (!file.exists())
			try {
				file.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
				return;
			}
		
		BufferedOutputStream bufferedOutputStream = null;
		
		try {
			bufferedOutputStream = new BufferedOutputStream(new FileOutputStream(file));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return;
		}
		
		try {
			session.exportDocumentView(nodePath, bufferedOutputStream, true, false);
		} catch (PathNotFoundException e) {
			e.printStackTrace();
			return;
		} catch (IOException e) {
			e.printStackTrace();
			return;
		} catch (RepositoryException e) {
			e.printStackTrace();
			return;
		}
		
		try {
			bufferedOutputStream.close();
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}
		
	}

	public static void exportSysView (String nodePath, Session session, String fileAddress) {
		
		File file = new File(fileAddress);
		
		if (!file.exists())
			try {
				file.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
				return;
			}
		
		BufferedOutputStream bufferedOutputStream = null;
		
		try {
			bufferedOutputStream = new BufferedOutputStream(new FileOutputStream(file));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return;
		}
		
		try {
			session.exportSystemView(nodePath, bufferedOutputStream, true, false);
		} catch (PathNotFoundException e) {
			e.printStackTrace();
			return;
		} catch (IOException e) {
			e.printStackTrace();
			return;
		} catch (RepositoryException e) {
			e.printStackTrace();
			return;
		}
		
		try {
			bufferedOutputStream.close();
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}
		
	}
	
	public static String find (Session session, String expression, String language) {
QueryManager queryManager = null;
		
		String results = "";

		try {
			queryManager = session.getWorkspace().getQueryManager();
		} catch (RepositoryException e) {
			e.printStackTrace();
		}
		
		Query query = null;
		
		try {
			query = queryManager.createQuery(expression, language);
		} catch (InvalidQueryException e) {
			e.printStackTrace();
		} catch (RepositoryException e) {
			e.printStackTrace();
		}
		
		QueryResult queryResult = null;
		
		try {
			queryResult = query.execute();
		} catch (RepositoryException e) {
			e.printStackTrace();
		}
		
		NodeIterator nodeIterator = null;
		
		try {
			nodeIterator = queryResult.getNodes();
		} catch (RepositoryException e) {
			e.printStackTrace();
		}
		
		while (nodeIterator.hasNext()) {
			
			Node node = nodeIterator.nextNode();
			String nodeName = "";
			
			try {
				nodeName = node.getName();
			} catch (RepositoryException e) {
				e.printStackTrace();
			}
			
			results = results + "<br/>" + nodeName;
		}
		
		return results;
		
	}
	
	public static boolean saveFile (Node node, File file, String mimeType) {
		
		FileInputStream in;
		
		Node fileNode = null;
		
		// CRIA UM NÓ COM O NOME DO ARQUIVO
		
		try {
			fileNode = node.addNode(file.getName(), "nt:file");
		} catch (Exception e1) {
			e1.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e) {
				e.printStackTrace();
			}
			return false;
		}
		
		// CRIA UM NÓ DENTRO DO NÓ DE ARQUIVO PARA GUARDAR O PROPRIEDADE BINÁRIA
		
		Node jcrContent = null;
		
		try {
			jcrContent = fileNode.addNode("jcr:content", "nt:resource");
		} catch (Exception e1) {
			e1.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e) {
				e.printStackTrace();
			}
			return false;
		}
		
		// CRIA UM FILEINPUTSTREAM A PARTIR DO ARQUIVO PASSADO
		
		try {
			in = new FileInputStream(file);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			return false;
		}
		
		// CRIA UMA PROPRIEDADE CONTENDO O ARQUIVO PASSADO
		
		Calendar calendar = new GregorianCalendar();
		
		try {
			jcrContent.setProperty("jcr:data", in);
			in.close();
			jcrContent.setProperty("jcr:lastModified", calendar); 
			jcrContent.setProperty("jcr:mimeType", mimeType);
		} catch (Exception e) {
			e.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			return false;
		}
		
		// SALVA TODAS AS ALTERAÇÕES
		
		try {
			fileNode.getSession().save();
			jcrContent.getSession().save();
		} catch (Exception e) {
			e.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			return false;
		}
		
		return true;
		
	}

	public static boolean saveFile (Node node, byte[] data, String name, String mimeType) {
		
		ByteArrayInputStream in;
		
		Node fileNode = null;
		
		// CRIA UM NÓ COM O NOME DO ARQUIVO
		
		try {
			fileNode = node.addNode(name, "nt:file");
		} catch (Exception e1) {
			e1.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e) {
				e.printStackTrace();
			}
			return false;
		}
		
		// CRIA UM NÓ DENTRO DO NÓ DE ARQUIVO PARA GUARDAR O PROPRIEDADE BINÁRIA
		
		Node jcrContent = null;
		
		try {
			jcrContent = fileNode.addNode("jcr:content", "nt:resource");
		} catch (Exception e1) {
			e1.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e) {
				e.printStackTrace();
			}
			return false;
		}
		
		// CRIA UM FILEINPUTSTREAM A PARTIR DO ARQUIVO PASSADO
		
		try {
			in = new ByteArrayInputStream(data);
		} catch (Exception e) {
			e.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			return false;
		}
		
		// CRIA UMA PROPRIEDADE CONTENDO O ARQUIVO PASSADO
		
		Calendar calendar = new GregorianCalendar();
		
		try {
			jcrContent.setProperty("jcr:data", in);
			jcrContent.setProperty("jcr:lastModified", calendar); 
			jcrContent.setProperty("jcr:mimeType", mimeType);
		} catch (Exception e) {
			e.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			return false;
		}
		
		// SALVA TODAS AS ALTERAÇÕES
		
		try {
			fileNode.getSession().save();
			jcrContent.getSession().save();
		} catch (Exception e) {
			e.printStackTrace();
			try {
				node.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			return false;
		}
		
		return true;
		
	}
	
	public static boolean getFile (Node node, String directoryPath) {
		
		try {
			directoryPath = directoryPath + "/" + node.getName();
			System.out.println("PATH TO THE FILE THAT WILL BE CREATED: " + directoryPath);
		} catch (RepositoryException e3) {
			e3.printStackTrace();
			return false;
		}
		
		File file = new File(directoryPath);
		
		FileOutputStream out = null;
		
		InputStream in = null;
		
		BufferedOutputStream bufferedOutput = null;
		
		BufferedInputStream bufferedInput = null;
		
		Property property = null;
		
		try {
			property = node.getProperty("jcr:content/jcr:data");
		} catch (PathNotFoundException e2) {
			e2.printStackTrace();
			return false;
		} catch (RepositoryException e2) {
			e2.printStackTrace();
			return false;
		}
		
		if (!file.exists()) {
			
			try {
				file.createNewFile();
			} catch (IOException e1) {
				e1.printStackTrace();
				return false;
			}
			
			try {
				in = property.getStream();
			} catch (ValueFormatException e1) {
				e1.printStackTrace();
				return false;
			} catch (RepositoryException e1) {
				e1.printStackTrace();
				return false;
			}
							
			try {
				out = new FileOutputStream(file);
			} catch (FileNotFoundException e) {
				e.printStackTrace();
				return false;
			}
			
			bufferedInput = new BufferedInputStream(in);
			
			bufferedOutput = new BufferedOutputStream(out);
			
			int input = 0;
			
			try {
				input = bufferedInput.read();
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}
			
			while (input != -1) {
			
				try {
					bufferedOutput.write(input);
					input = bufferedInput.read();
				} catch (IOException e) {
					e.printStackTrace();
					return false;
				}

			}
			
			try {
				bufferedOutput.close();
				bufferedInput.close();
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}
			
			return true;
			
		} else {
			return false;
		}
	}
	
	public static BufferedInputStream getFile (Node node) {
				
		InputStream in = null;
		
		BufferedInputStream bufferedInput = null;
		
		Property property = null;
		
		try {
			property = node.getProperty("jcr:content/jcr:data");
		} catch (PathNotFoundException e2) {
			e2.printStackTrace();
			return null;
		} catch (RepositoryException e2) {
			e2.printStackTrace();
			return null;
		}
		
		try {
			in = property.getStream();
		} catch (ValueFormatException e1) {
			e1.printStackTrace();
			return null;
		} catch (RepositoryException e1) {
			e1.printStackTrace();
			return null;
		}
								
		bufferedInput = new BufferedInputStream(in);
		
		return bufferedInput;
	}

}
